/*   -= Minimal VFD Clock v4.0 =-
 *   
 *      attiny85  - 
 *   RTC - DS3231
 *    
 *    ":",  
 *    
 *     
 *   
 *     ,    
 */

#include <TinyWireM.h>

#define dataPin 1
#define clockPin 4
#define latchPin 3
#define buttonPin 0

unsigned long getTimer, butTimer;    // 
volatile byte pos, digit;
byte mode;
boolean butState, butFlag, butPres, butRel, butHold, butHO;   //  
byte second, minute, hour;

//  (     , 1 -  !)
byte digits[11]   = { 0b00000001,   // = "0"    
                      0b01101011,   // = "1"
                      0b00100100,   // = "2"
                      0b00100010,   // = "3"
                      0b01001010,   // = "4"
                      0b00010010,   // = "5"
                      0b00010000,   // = "6"
                      0b00101011,   // = "7"
                      0b00000000,   // = "8"
                      0b00000010,   // = "9"
                      0b01111111    // =  
                    };

//  (     , 1 -  !)                    
byte grids[5]     = { 0b01111000,   // . .
                      0b01110100,   // . .
                      0b01101100,   // ":"
                      0b01011100,   // . .
                      0b00111100   // . .
                    };

/////  ..
byte decToBcd(byte val){
  return ( (val/10*16) + (val%10) );
}

byte bcdToDec(byte val){
  return ( (val/16*10) + (val%16) );
}

void setDate(byte second,        // 0-59
             byte minute,        // 0-59
             byte hour)          // 1-23
{
   TinyWireM.beginTransmission(0x68);
   TinyWireM.send(0);
   TinyWireM.send(decToBcd(second));    
   TinyWireM.send(decToBcd(minute));
   TinyWireM.send(decToBcd(hour));     
   TinyWireM.endTransmission();
}

void getDate(byte *second,
             byte *minute,
             byte *hour)
{
  TinyWireM.beginTransmission(0x68);
  TinyWireM.send(0);
  TinyWireM.endTransmission();

  TinyWireM.requestFrom(0x68, 7);
  *second     = bcdToDec(TinyWireM.receive() & 0x7f);
  *minute     = bcdToDec(TinyWireM.receive());
  *hour       = bcdToDec(TinyWireM.receive() & 0x3f); 
}
/////////////////

void setup() {
  TCCR1 |= (1 << CTC1);                               //    1
  TCCR1 |= (1 << CS12) | (1 << CS11) | (1 << CS10);   //  64
  OCR1C = 63;                                         //  
  TIMSK |= (1 << OCIE1A);                             //   
  
  TinyWireM.begin();
  
  pinMode(clockPin, OUTPUT);
  pinMode(dataPin, OUTPUT);
  pinMode(latchPin, OUTPUT);
  digitalWrite(latchPin, HIGH);  
}


//  ( ,   )
SIGNAL(TIMER1_COMPA_vect) {  //  
  pos++;
  if (pos == 5) pos = 0;
  switch (pos) {
    case 0:
      if (mode == 2) { digit = digits[10]; }
      else { digit = digits[minute%10]; }
      break;
    case 1:
      if (mode == 2) { digit = digits[10]; }
      else { digit = digits[minute/10]; }
      break;
    case 2:
      if (second%2 & mode == 0) { digit = digits[10]; }
      else { digit = digits[1]; }
      break;
    case 3:
      if (mode == 1) { digit = digits[10]; }
      else { digit = digits[hour%10]; }
      break;
    case 4:
      if (hour/10 == 0 || mode == 1) { digit = digits[10]; }
      else { digit = digits[hour/10]; }
      break;
  }
  digitalWrite(latchPin, LOW);
  //shiftOut(dataPin, clockPin, LSBFIRST, digit);
  shiftOut(dataPin, clockPin, LSBFIRST, grids[pos]);
  shiftOut(dataPin, clockPin, LSBFIRST, digit);
  digitalWrite(latchPin, HIGH); 
}

void loop() {

//     
  if (mode == 0 & millis() - getTimer >= 100) { //     
    getTimer = millis();
    getDate(&second, &minute, &hour);
  }
  
//  
  int butVal = analogRead(buttonPin);
  if (butVal < 950) { butState = 1; } 
  else { butState = 0; }
  if (butState && !butFlag && millis() - butTimer > 10) {    //  ( )
    butFlag = 1;
    butHO = 1;
    butTimer = millis();
  }  
  if (!butState && butFlag && millis() - butTimer < 500) {   //    hold,  
    butPres = 1;
    butFlag = 0;
  }  
  if (butFlag && !butHold && millis() - butTimer > 500) {    //    hold,   
    butHold = 1;
  }
  if (!butState && butFlag && millis() - butTimer > 500) {   //    hold,  ,   
    butFlag = 0;
    butHold = 0;
    butTimer = millis();
  }
  
  if (butPres) {    //   1  -  /
    if (mode == 1) {
      minute++;
      if (minute == 60) minute = 0;
    } else if (mode == 2) {
      hour++;
      if (hour == 24) hour = 0;
    }
    butPres = 0;
  }
  if (butHold && butHO) {   //    - :   -   -  
    mode++;
    if (mode == 3) {
      second = 0;
      setDate(second, minute, hour); 
      mode = 0;
    }
    butHO = 0;
  }  
}